!+
! Module: rgslinepos_mod
!
! Authors:  C. Erd - XMM-Newton SOC
!          
! Description: Calculate the position of a center of an emission line on the RGS camera
! 
! Comments: 
! 
! $Id: rgslinepos_mod.f90,v 1.3 2001/10/26 13:09:03 cerd Exp $
!- 
module rgslinepos_mod

  use dal
  use param  
  use constants
  use errorHandling
  use cal
  use pointing_mod
  use vector_mod

  implicit none 
  
  private 

!  type SpacecraftAttitudeType
!     real(double) :: ra, dec, apos
!  end type SpacecraftAttitudeType

  public rgslinepos

contains 
  
  !---------------------------------------------------------------------
  subroutine rgslinepos
  !---------------------------------------------------------------------

                                !from rgslib, pointing_mod
!    type(SpacecraftAttitudeType)  :: Attitude 
!    type(SkyPositionType)         :: SrcPos
!    real (double),dimension(2)    :: angles   ! dispersion and x-disp direction

    integer(kind=int16), dimension(:), pointer :: chipX => null(), chipY => null()         !CHIPCOORD
    integer(kind=int8) , dimension(:), pointer :: ccds => null()                 !CHIPCOORD
    real(kind=double)  , dimension(:), pointer :: xmm1 => null(), ymm1 => null(), zmm1 => null()     !CAMCOORD1
    real(kind=double)  , dimension(:), pointer :: xmm2 => null(), ymm2 => null(), zmm2 => null()     !CAMCOORD2
    real(kind=double)  , dimension(:), pointer :: xmmSc => null(), ymmSc => null(), zmmSc => null()  !SACCORD
    real(kind=double)  , dimension(:), allocatable :: beta             !ROWCOORD
    real(kind=double)  , dimension(:,:), allocatable :: chi            !ROWCOORD
!    real(kind=single)  , dimension(:), pointer :: betap, chip          !ROWCCORD

    real(kind=double), dimension(2) :: alpha
    character(len=4), dimension(2) :: instr= (/ 'rgs1', 'rgs2' /)
    integer :: i, j
    real(kind=double) :: deltadisp, FoverL
    real(kind=double), dimension(:), allocatable :: wavelength

    allocate( beta(parameterCount("wavelength")) )
    allocate(  chi(parameterCount("wavelength"),2) )
    allocate( wavelength(parameterCount("wavelength")) )

    do i=1,parameterCount("wavelength")
       wavelength(i) = realParameter("wavelength", i-1)
    end do

                                !setup alpha for both instruments
!    if ( booleanParameter("withcelestcoord") ) then
!                                !input is with celestial coordinates
!       Attitude%ra   = realParameter("attra")   *acos(-1.d0)/180.
!       Attitude%dec  = realParameter("attdec")  *acos(-1.d0)/180.
!       Attitude%apos = realParameter("attapos") *acos(-1.d0)/180.
!
!       SrcPos%ra  = realParameter("srcra")  *acos(-1.d0)/180.
!       SrcPos%dec = realParameter("srcdec") *acos(-1.d0)/180.
!
!       call CAL_setState(instrument=instId(stringParameter("primeinstrument")))
!       angles = offAxisAngles(Attitude, SrcPos)
!
!       if ( stringParameter("primeinstrument") == instr(1) ) then
!          alpha(1) = angles(1)
!          chi(:,1) = angles(2)    !cross-dispersion direction
!
!          i = 2                 !setup for the other instrument
!       else
!          alpha(2) = angles(1)
!          chi(:,2) = angles(2)    !cross-dispersion direction
!
!          i = 1                 !setup for the other instrument
!       end if
!
!!       Attitude =               !pointing of the other instrument
!
!       call CAL_setState(instrument=instId(instr(i)))
!       angles = offAxisAngles(Attitude, SrcPos)
!       alpha(i) = angles(1)
!       chi(:,i) = angles(2)
!
!    else                        !only off axis angle is supplied

       deltadisp = realParameter("deltadisp") /60.d0/180.d0*acos(-1.d0) !arcmin -> rad
       chi = 0.
!    endif

       if ( stringParameter("instrument") == instr(1) ) then
          i = 1
       else
          i = 2
       end if

!    Instruments: do i=1,2       !get line position for both instruments
       call message( appMsg, Always, trim(stringParameter("instrument")) )
       call CAL_setState(instrument=instId(stringParameter("instrument")), &
            ocb=intParameter("ocb"), randomize=.false.)

       FoverL = CAL_getMiscellaneousDataValue('FOCAL_LENGTH') / norm( &
            (/ CAL_getMiscellaneousDataValue("LINCOORD:LINCOORD%G_TX"), &
               CAL_getMiscellaneousDataValue("LINCOORD:LINCOORD%G_TY"), &
               CAL_getMiscellaneousDataValue("LINCOORD:LINCOORD%G_TZ") /) - &
            (/ CAL_getMiscellaneousDataValue("LINCOORD:LINCOORD%CC2_TX"), &
               CAL_getMiscellaneousDataValue("LINCOORD:LINCOORD%CC2_TY"), &
               CAL_getMiscellaneousDataValue("LINCOORD:LINCOORD%CC2_TZ") /) )

       deltadisp = deltadisp * FoverL !convert off-axis angle to delta_alpha

       alpha(i) = deltadisp + ( CAL_getMiscellaneousDataValue('INCIDENCE_ANGLE') + &
         CAL_getMiscellaneousDataValue("LINCOORD:LINCOORD%G_RY") ) /180.d0*acos(-1.d0)

                                !grating equation gives dispersion angle
       beta = acos( -dble(abs(intParameter("order"))) * wavelength / &
            CAL_getMiscellaneousDataValue('GRAT_LINE_DENS') + &
            cos(alpha(i)) )

       write(errstr,*) 'ALPHA=',alpha(i)
       call message(appMsg,Verbose)
       write(errstr,*) 'BETA=',beta
       call message(appMsg,Verbose)

                                !sequence of calls to convert from ROWCOORD to CHIPCOORD
       call CAL_rowCoordToSacCoord(beta, chi(:,i), xmmSc, ymmSc, zmmSc)
       write(errstr,*) 'xmmSc=', (xmmSc(i), i=0,size(xmmSc)-1)
       write(errstr,*) 'ymmSc=', (ymmSc(i), i=0,size(ymmSc)-1)
       write(errstr,*) 'zmmSc=', (zmmSc(i), i=0,size(zmmSc)-1)
       call message(appMsg,Verbose)

       call CAL_sacCoordToCamCoord2(xmmSc, ymmSc, zmmSc, xmm2, ymm2, zmm2)
       write(errstr,*) 'xmm2=', (xmm2(i), i=0,size(xmm2)-1)
       write(errstr,*) 'ymm2=', (ymm2(i), i=0,size(ymm2)-1)
       write(errstr,*) 'zmm2=', (zmm2(i), i=0,size(zmm2)-1)
       call message(appMsg,Verbose)

       call CAL_camCoord2ToCamCoord1(xmm2, ymm2, zmm2, xmm1, ymm1, zmm1)
       write(errstr,*) 'xmm1=', (xmm1(i), i=0,size(xmm1)-1)
       write(errstr,*) 'ymm1=', (ymm1(i), i=0,size(ymm1)-1)
       write(errstr,*) 'zmm1=', (zmm1(i), i=0,size(zmm1)-1)
       call message(appMsg,Verbose)

       call CAL_camCoord1ToChipCoord(xmm1, ymm1, zmm1, chipX, chipY, ccds)
       write(errstr,*) 'ccds=', (ccds(i),   i=0,size(ccds)-1)
       write(errstr,*) 'chipX=', (chipX(i), i=0,size(chipX)-1)
       write(errstr,*) 'chipY=', (chipY(i), i=0,size(chipY)-1)
       call message(appMsg,Verbose)

       Wavelengths: do j=1,size(wavelength) !write locations of all wavelengths
          if ( ccds(j-1) >9 .or. ccds(j-1) <1 ) then
             write (errstr,*) wavelength(j),'A: in chip gap or off camera'
          else
             write (errstr,*) wavelength(j),'A: CCD=',ccds(j-1),' chipX=',chipX(j-1),',', &
                  min( int(chipX(j-1)), ceiling(512./real(intParameter("ocb")))-chipX(j-1) ), &
                  'bins from inter-chip boundary'
          end if
          call message( appMsg, Always )
       end do Wavelengths

!       print*,'****** start debugging' !this is for debugging purposes only
!       chipX = 150
!       chipY = 60
!       call CAL_setState1(ccdChipId=5)
!       call CAL_chipCoordToCamCoord1(chipX, chipY, xmm1, ymm1, zmm1)
!       call CAL_camCoord1ToCamCoord2(xmm1, ymm1, zmm1, xmm2, ymm2, zmm2)
!       call CAL_camCoord2ToSacCoord(xmm2, ymm2, zmm2, xmmSc, ymmSc, zmmSc)
!       call CAL_sacCoordToRowCoord(xmmSc, ymmSc, zmmSc, betap, chip)
!       print*, 'chipX=', chipX, ', chipY=', chipY
!       print*, 'xmm1=', xmm1, ', ymm1=', ymm1, ', zmm1=', zmm1
!       print*, 'xmm2=', xmm2, ', ymm2=', ymm2, ', zmm2=', zmm2
!       print*, 'xmmSc=', xmmSc, ', ymmSc=', ymmSc, ', zmmSc=', zmmSc
!       print*, 'beta=',betap,', chi=',chip
!
!       beta = betap
!       chi(:,i) = chip
!
!       call CAL_rowCoordToSacCoord(beta, chi(:,i), xmmSc, ymmSc, zmmSc)
!       call CAL_sacCoordToCamCoord2(xmmSc, ymmSc, zmmSc, xmm2, ymm2, zmm2)
!       call CAL_camCoord2ToCamCoord1(xmm2, ymm2, zmm2, xmm1, ymm1, zmm1)
!       call CAL_camCoord1ToChipCoord(xmm1, ymm1, zmm1, chipX, chipY, ccds)
!       print*, 'xmmSc=', xmmSc, ', ymmSc=', ymmSc, ', zmmSc=', zmmSc
!       print*, 'xmm2=', xmm2, ', ymm2=', ymm2, ', zmm2=', zmm2
!       print*, 'xmm1=', xmm1, ', ymm1=', ymm1, ', zmm1=', zmm1
!       print*, 'ccds=', ccds, ', chipX=', chipX, ', chipY=', chipY
                                !end of debugging
!    end do Instruments
  end subroutine rgslinepos
  
end module rgslinepos_mod
